// Dlg_Common.cpp
// Neuron Computing Inc.
#include "Dlg_Common.h"

#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>

extern XApp* g_app;

// Warning
void messageBox(char* text)
{
	Dlg_Info dlg;
	dlg.m_parent = g_app->m_mainFrame->m_w;
	dlg.show_info(text);
}

// If the following flag is set, then popup dialog boxes will
// automatically go away if the mouse cursor leaves the popup window.
#define POPUP_GOAWAY 0

Bool isAncestorOf(Window child, Widget parent)
{
/*
	Window windowP = XtWindow(parent);
	if (child == windowP)
		return True;
	Display* d = XtDisplay(parent);
	Window root;
	Window p;
	Window* children;
	unsigned int children_count;
	XQueryTree(d, child, &root, &p, &children, &children_count);
	if (p == XtWindow(parent))
		return True;
	return False;
*/
	Display* d = XtDisplay(parent);
	Widget w = XtWindowToWidget(d, child);
	while (w != NULL)
	{
		if (w == parent)
			return True;
		w = XtParent(w);
	}
	return FALSE;
}

/////////////////////////////////////////////////////////////////
// Event handlers
static void command_ok(Widget w, Dlg_Base* dlg, XtPointer client_data)
{
	dlg->action(Dlg_Base::IDOK);
}

static void command_cancel(Widget w, Dlg_Base* dlg, XtPointer client_data)
{
	dlg->action(Dlg_Base::IDCANCEL);
}

/////////////////////////////////////////////////////////////////
// Dlg_Base
Dlg_Base::Dlg_Base()
{
	m_parent = NULL;
	m_done = FALSE;
	m_shell = NULL;
	m_form = NULL;
	m_result = IDCANCEL;
}

Bool Dlg_Base::createForm()
{
	if (g_app->m_popup_active == 1)
		return FALSE;
	g_app->m_popup_active = 1;

	Arg args[2];
	int n = 0;
#ifndef MOTIF
	XtSetArg(args[n], XtNlabel, "popup_shell_label"); n++;
	m_shell = XtCreatePopupShell("popup_shell", POPUP_SHELL, // DIALOG
									m_parent, args, n);
#else
	XtSetArg(args[n], XmNautoUnmanage, FALSE); n++;
	m_shell = XmCreateBulletinBoardDialog(m_parent, "Edit", args, n);
#endif
	m_form = XtCreateManagedWidget("form", FORM, m_shell, NULL, 0);
	return TRUE;
}

void Dlg_Base::createOkCancel(int x, int y)
{
	Arg args[4];
	int n = 0;
	// OK
	n = 0;
	XtSetArg(args[n], XtNx, x); n++;
	XtSetArg(args[n], XtNy, y); n++;
	XtSetArg(args[n], XtNwidth, 80); n++;
	Widget ok_button = XtCreateManagedWidget("OK", COMMAND, m_form, args, n);
	// Cancel
	n = 0;
	XtSetArg(args[n], XtNx, x+80+10); n++;
	XtSetArg(args[n], XtNy, y); n++;
	XtSetArg(args[n], XtNwidth, 80); n++;
	Widget cancel_button = XtCreateManagedWidget("Cancel", COMMAND, m_form, args, n);
#ifndef MOTIF
	XtAddCallback(ok_button, XtNcallback,
					(XtCallbackProc)command_ok, (XtPointer)this);
	XtAddCallback(cancel_button, XtNcallback,
					(XtCallbackProc)command_cancel, (XtPointer)this);
#else
	XtAddCallback(ok_button, XmNactivateCallback,
					(XtCallbackProc)command_ok, (XtPointer)this);
	XtAddCallback(cancel_button, XmNactivateCallback,
					(XtCallbackProc)command_cancel, (XtPointer)this);
#endif
}

void Dlg_Base::doModal()
{
	while (!m_done)
	{
		XEvent e;
//		if (isAncestorOf(e.xany.window, m_shell) || e.type == Expose)
		{
			XtAppNextEvent(g_app->m_appContext, &e);
			XtDispatchEvent(&e);
		}
	}
}

void Dlg_Base::endDialog(int id)
{
	m_done = true;
	m_result = id;
}

void Dlg_Base::action(int id)
{
	if (id == IDOK)
		endDialog(id);
	else if (id == IDCANCEL)
		endDialog(id);
}

void Dlg_Base::destroyForm()
{
#ifndef MOTIF
	XtPopdown(m_shell);
	XtDestroyWidget(m_shell);
#else
	XtUnmanageChild(m_shell);
#endif
	g_app->m_popup_active = 0;
}

/////////////////////////////////////////////////////////////////
//  Data and code for show_info method:

void Dlg_Info::show_info(char * prompt)
{
	if (!createForm())
		return;

	Arg args[5];
	// Label
	int n = 0;
	XtSetArg(args[n], XtNy, 2); n++;
	XtSetArg(args[n], XtNlabel, prompt); n++;

	Widget label = XtCreateManagedWidget(prompt, LABEL, m_form, args, n);
	// Button
#ifdef MOTIF
	n = 0;
	XtSetArg(args[n], XtNy, 40); n++;
	XtSetArg(args[n], XtNwidth, 80); n++;
	Widget ok_button = XtCreateManagedWidget("OK", COMMAND, m_form, args, n);
	XtAddCallback(ok_button, XmNactivateCallback,
					(XtCallbackProc)command_ok, (XtPointer)this);
#else
	n = 0;
	XtSetArg(args[n], XtNy, 110); n++;
	XtSetArg(args[n], XtNwidth, 80); n++;
	XtSetArg(args[n], XtNfromVert, label); n++;
	XtSetArg(args[n], XtNvertDistance, 20); n++;
	Widget ok_button = XtCreateManagedWidget("OK", COMMAND, m_form, args, n);
	XtAddCallback(ok_button,XtNcallback,
					(XtCallbackProc)command_ok, (XtPointer)this);
#endif

	XtManageChild(m_shell);

	// Warp pointer to edit field
#ifndef MOTIF
	XWarpPointer(XtDisplay(ok_button), None, XtWindow(ok_button),
					0, 0, 0, 0, 10, 10);
	XSync(XtDisplay(ok_button), False);
#endif

	doModal();
	destroyForm();
}

///////////////////////////////////////////////////////////////////
// List selection:

#ifndef MOTIF
void list_selected(Widget this_list, Dlg_List* dlg, XtPointer p)
{
	XawListReturnStruct *list_struct = XawListShowCurrent(this_list);
	dlg->itemSelected(list_struct->list_index);
	free((char*)list_struct); // we need to give back this storage
}
#else
void list_selected(Widget, Dlg_List* dlg, XmListCallbackStruct* list)
{
	dlg->itemSelected(list->item_position);
}
#endif

// Dlg_List
Dlg_List::Dlg_List()
{
	m_curSel = -1;
}

void Dlg_List::action(int id)
{
	endDialog(id);
}

void Dlg_List::itemSelected(int index)
{
	m_curSel = index;
}

int Dlg_List::choose_one_from_list(char *prompt, char **list_items,
                                     int number_of_items)
{
	if (!createForm())
		return 0;

	int n;
	Arg args[10];
	// Label
	n = 0;
	XtSetArg(args[n], XtNy, 2); n++;
#ifndef MOTIF
	XtSetArg(args[n], XtNlabel, prompt); n++;
#endif
	Widget label = XtCreateManagedWidget(prompt, LABEL, m_form, args, n);

	n = 0;

#ifndef MOTIF
	// List
	XtSetArg(args[n], XtNfromVert, label); n++;
	XtSetArg(args[n], XtNvertDistance, 10); n++;
	XtSetArg(args[n],XtNlist,list_items);  n++;
	XtSetArg(args[n],XtNnumberStrings,number_of_items);  n++;
	int num_columns = 1 + (number_of_items / 30);
	XtSetArg(args[n],XtNdefaultColumns,num_columns);  n++;
	XtSetArg(args[n],XtNforceColumns,True); n++;
	Widget a_list = XtCreateManagedWidget("a_list", LIST, m_form, args, n);
	XtAddCallback(a_list, XtNcallback,
					(XtCallbackProc)list_selected, (XtPointer)list_selected);
	// Cancel button
	n = 0;
	XtSetArg(args[n], XtNwidth, 80); n++;
	XtSetArg(args[n], XtNfromVert, a_list); n++;
	XtSetArg(args[n], XtNvertDistance, 10); n++;
	Widget button_2 = XtCreateManagedWidget("Cancel", COMMAND, m_form, args, n);
	XtAddCallback(button_2, XtNcallback, (XtCallbackProc)command_cancel, 
					(XtPointer)this);
	XtManageChild(m_shell);

	// Warp pointer to edit field
	XWarpPointer(XtDisplay(a_list),None,XtWindow(a_list),
					0, 0, 0, 0, 40, 40);
	XSync(XtDisplay(a_list),False);
#else
	XmString* stringXm = (XmString*)XtMalloc(
								sizeof(XmString)*(number_of_items + 1));
	int k;
	for (k=0; k<number_of_items; k++)
		stringXm[k] = XmStringCreate(list_items[k], XmSTRING_DEFAULT_CHARSET);

	printf("size of list=%d\n", number_of_items);
	// List
	XtSetArg(args[n], XmNitems, stringXm); n++;
	XtSetArg(args[n], XmNitemCount, number_of_items); n++;
	int visible_count = number_of_items;
	if (visible_count > 16)
		visible_count = 16;
	XtSetArg(args[n], XmNvisibleItemCount, visible_count); n++;
	XtSetArg(args[n], XtNy, 50); n++;
	Widget a_list = XtCreateManagedWidget("a_list", LIST, m_form, args, n);
	XtAddCallback(a_list, XmNbrowseSelectionCallback,
					(XtCallbackProc)list_selected, (XtPointer)this);

	for (k=0; k<number_of_items; k++)
		XmStringFree(stringXm[k]);
	free((char*)stringXm);
	// Cancel button
	n = 0;
	XtSetArg(args[n], XtNy, 20); n++;
	XtSetArg(args[n], XtNwidth, 80); n++;
	Widget button_2 = XtCreateManagedWidget("Cancel", COMMAND, m_form, args, n);
	XtAddCallback(button_2, XmNactivateCallback,
					(XtCallbackProc)command_cancel, (XtPointer)this);

	XtManageChild(a_list);
	XtManageChild(m_form);
	XtManageChild(m_shell);
#endif

	// Event loop
	doModal();
	// Exit
	destroyForm();

	return m_curSel;
}

////////////////////////////////////////////////////////////////
//  Data and code for do_edit method:

#ifndef MOTIF
	#define XtDialogGetValueString XawDialogGetValueString
	#define XtDialogFreeValueString
#else
	#define XtDialogGetValueString XmTextGetString
	#define XtDialogFreeValueString free
#endif

void set_leave_widget_callback(Widget, XtActionProc);

void Dlg_Edit::action(int id)
{
	if (id == IDOK)
	{
		char* cp = XtDialogGetValueString(m_textField);
		XtDialogFreeValueString(cp);
		endDialog(id);
	}
	else if (id == IDCANCEL)
	{
		m_buffer[0] = 0;
		endDialog(id);
	}
}

void Dlg_Edit::do_edit(char* prompt, char* returned_string)
{
	if (!createForm())
		return;

	strcpy(m_buffer, returned_string);

	Arg args[10];
	int n = 0;
#ifndef MOTIF
	// Text Field
	XtSetArg(args[n], XtNlabel, prompt); n++;
	XtSetArg(args[n], XtNvalue, m_buffer); n++;
	XtSetArg(args[n], XtNwidth, 560); n++;
	m_textField = XtCreateManagedWidget("dialog", TEXT_FIELD, m_form, args, n);

	createOkCancel(10, 560);
#else
	// Label
	XtSetArg(args[n], XtNlabel, prompt); n++;
	Widget label = XtCreateManagedWidget(prompt, LABEL, m_form, args, n);
	// Text Field
	n = 0;
	XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
	XtSetArg(args[n], XmNwordWrap, TRUE); n++;
	XtSetArg(args[n], XmNrows, 10); n++;
	XtSetArg(args[n], XmNcolumns, 60); n++;
	XtSetArg(args[n], XmNheight, 90); n++;
	XtSetArg(args[n], XmNy, 20); n++;
	XtSetArg(args[n], XmNx, 10); n++;
	XtSetArg(args[n], XmNvalue, returned_string); n++;
	m_textField = XtCreateManagedWidget("dialog", TEXT_FIELD, m_form, args, n);
	createOkCancel(10, 120);
#endif

#if POPUP_GOAWAY
	set_leave_widget_callback(m_shell, (XtActionProc)do_edit_cancel);
#endif

	XtManageChild(m_shell);

	// Warp pointer to edit field
#ifndef MOTIF
	XWarpPointer(XtDisplay(m_textField),None,XtWindow(m_textField),
					0, 0, 0, 0, 40, 40);
	XSync(XtDisplay(m_form), False);
#endif

	doModal();
	destroyForm();

	// Copy the edited string back into the returned_string buffer
	// (the caller must have allocated sufficient space for this):
	strcpy(returned_string, m_buffer);
}

////////////////////////////////////////////////////////////////
// Dlg_ChooseFont
int Dlg_ChooseFont::choose_font()
{
	int count;
	char** fontList = XListFonts(XtDisplay(m_parent), "*", 100, &count);
	if (count <= 0)
		return 0;
	Dlg_List dlg;
	dlg.m_parent = m_parent;
	dlg.choose_one_from_list("Font", fontList, count);
	XFreeFontNames(fontList);
	return 1;
}
