////////////////////////////////////////////////////////////////
// Sample java program for Neuron Visual Java
// Neuron Computing Inc.

import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Insets;

import java.awt.Container;
import java.awt.Window;
import java.awt.Frame;

import java.awt.Dialog;
import java.awt.FileDialog;

import java.awt.TextField;
import java.awt.TextArea;
import java.awt.Button;
import java.awt.ScrollPane;
import java.awt.Label;
import java.awt.Scrollbar;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.CardLayout;

import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.Menu;
import java.awt.MenuBar;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;

import java.awt.Font;

import neuron.DialogLayout;
import neuron.JFormContainer;

////////////////////////////////////////////////////////////////
// MyDialog
class MyDialog extends DialogLayout
{
	String m_text = "text";

MyDialog()
{
	m_form.setTitle("MyDialog");
	m_form.setFont(new Font("Sans Serif", 0, 11));

	m_form.addOkCancelButtons();
	m_form.addLabel("Test Label", Label.LEFT);
	m_form.addTextField(10);
	m_form.addChoice();
	add(4, "First");
	add(4, "Second");
	add(4, "Third");
	m_form.addButton("My Dialog", false, 5);
}

public void layout()
{
	m_form.setDefaultBounds(2, 0, 0, DialogLayout.leftBelow);
	m_form.alignRight(3, 2, 4);
	m_form.alignBelow(4, 3, 4);
	// Ok - Cancel
	m_form.setDefaultBounds(0, -8, 0, DialogLayout.leftBelow);
	m_form.alignBelow(1, 0, 4);
	m_form.alignBelow(5, 1, 4);
}

public void updateData(boolean save)
{
	if (save)
	{
		m_text = getText(3);
	}
	else
	{
		setText(3, m_text);
	}
}

public void processAction(int id)
{
	if (id == 5)
	{
		updateData(true);
		MyDialog dlg = new MyDialog();
		dlg.m_text = m_text;
		if (dlg.doModal((Frame)m_dialog.getOwner()) == DialogLayout.ok)
		{
			m_text = dlg.m_text;
			updateData(false);
		}
	}
}
}

////////////////////////////////////////////////////////////////
// MyClass
class MyClass extends WindowAdapter implements ActionListener
{
    public static void main(String args[])
	{
		MyClass myClass = new MyClass();
		myClass.init();
	}

	private Frame m_frame;
	TextArea m_textArea;

	public void init()
	{
		m_frame = new Frame("MyFrame");

		// Buttons
		Button loadButton = new Button("Open");
		loadButton.addActionListener(this);
		loadButton.setActionCommand("open");
		Button saveButton = new Button("Save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		Button gridLayoutButton = new Button("Grid Layout");
		gridLayoutButton.setActionCommand("gridL");
		gridLayoutButton.addActionListener(this);
		Button windowButton = new Button("Window");
		windowButton.setActionCommand("window");
		windowButton.addActionListener(this);

		// Frame
		BorderLayout layout = new BorderLayout(10, 10);
		m_frame.setLayout(layout);
		m_frame.add("North", loadButton);
		m_frame.add("South", saveButton);

		ScrollPane scrollPane = new ScrollPane();
		m_textArea = new TextArea();
		scrollPane.add("Center", m_textArea);
		m_frame.add("Center", scrollPane);

		m_frame.add("East", gridLayoutButton);
		m_frame.add("West", windowButton);
		m_frame.pack();
		m_frame.setBounds(200, 200, 500, 400);
		m_frame.addWindowListener(this);
		// Menu
		m_frame.setMenuBar(createMenuBar());
		m_frame.setVisible(true);

		Dimension dim = m_frame.getSize();
		Rectangle rect = m_frame.getBounds();
		Insets insets = m_frame.getInsets();
		m_textArea.append("Size: " + dim.width + ", " + dim.height + "\n");
		showRect("Bounds: ", rect);
		showInsets("Insets: ", insets);
    }

	void showRect(String name, Rectangle rect)
	{
		m_textArea.append(name + rect.x + ", " + rect.y + ", " + rect.width + ", " + rect.height + "\n");
	}

	void showString(String text)
	{
		m_textArea.append(text);
		m_textArea.append("\n");
	}

	void showInsets(String name, Insets insets)
	{
		m_textArea.append(name + insets.left + ", " + insets.right + ", " + insets.top + ", " +  + insets.bottom + "\n");
	}

	public MenuBar createMenuBar()
	{
		MenuBar menuBar = new MenuBar();
		// File
		Menu menu = new Menu("File");

		MenuItem menuItem = new MenuItem("Open");
		menuItem.setActionCommand("open");
		MenuShortcut shortcut = new MenuShortcut('O');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menuItem = new MenuItem("Save");
		menuItem.setActionCommand("save");
		shortcut = new MenuShortcut('S');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menuItem = new MenuItem("Exit");
		menuItem.setActionCommand("exit");
		menu.add(menuItem);

		menu.addActionListener(this);
		menuBar.add(menu);

		// Edit
		menu = new Menu("Edit", true);

		menu.addActionListener(this);
		menuBar.add(menu);

		// Menus
		menu = new Menu("Menus");

		menuItem = new MenuItem("Shell Menu");
		menuItem.setActionCommand("shellMenu");
		shortcut = new MenuShortcut('S');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menu.addActionListener(this);
		menuBar.add(menu);

		// Window
		menu = new Menu("Dialogs");

		menuItem = new MenuItem("My Dialog...");
		menuItem.setActionCommand("myDialog");
		shortcut = new MenuShortcut('M');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menuItem = new MenuItem("Signals Scale...");
		menuItem.setActionCommand("signalsScale");
		shortcut = new MenuShortcut('S');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menuItem = new MenuItem("Display Settings...");
		menuItem.setActionCommand("displaySettings");
		shortcut = new MenuShortcut('D');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menuItem = new MenuItem("Edit Source File...");
		menuItem.setActionCommand("editSourceFile");
		shortcut = new MenuShortcut('E');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);
		menu.addActionListener(this);

		menuBar.add(menu);

		// Window
		menu = new Menu("Window");

		menuItem = new MenuItem("Grid Layout Dialog");
		menuItem.setActionCommand("gridL");
		shortcut = new MenuShortcut('G');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menuItem = new MenuItem("Grid Bag Layout Dialog");
		menuItem.setActionCommand("gridB");
		shortcut = new MenuShortcut('B');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);

		menuItem = new MenuItem("Card Layout Dialog");
		menuItem.setActionCommand("cardL");
		shortcut = new MenuShortcut('C');
		menuItem.setShortcut(shortcut);
		menu.add(menuItem);
		menu.addActionListener(this);

		menuBar.add(menu);

		return menuBar;
	}

	void commandGridLayout()
	{
		// Dialog Grid
		Dialog dialog = new Dialog(m_frame, "Grid Layout Dialog");
		GridLayout layout = new GridLayout(3, 2, 10, 10);
		dialog.setLayout(layout);
		dialog.add(new Label("1"));
		dialog.add(new Button("2"));
		dialog.add(new Label("3"));
		dialog.add(new Button("4"));
		dialog.add(new Scrollbar(Scrollbar.VERTICAL, 5, 0, 0, 10));
		dialog.add(new Button("6"));
		dialog.setSize(300, 400);
		dialog.addWindowListener(this);
		dialog.setVisible(true);
	}

	void commandGridBagLayout()
	{
		Dialog dialog = new Dialog(m_frame, "Grid Bag Layout Dialog");

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		
		dialog.setFont(new Font("Helvetica", Font.PLAIN, 14));
		dialog.setLayout(gridbag);

		// Four buttons in the first row
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1.0;
		Button button = new Button("Button1");
		gridbag.setConstraints(button, c);
		dialog.add(button);

		button = new Button("Button2");
		gridbag.setConstraints(button, c);
		dialog.add(button);

		button = new Button("Button3");
		gridbag.setConstraints(button, c);
		dialog.add(button);
		
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		button = new Button("Button4");
		gridbag.setConstraints(button, c);
		dialog.add(button);

		// One button in second row
		c.weightx = 0.0;		   //reset to the default
		button = new Button("Button5"); //another row
		gridbag.setConstraints(button, c);
		dialog.add(button);

		// Two buttons in the third row
		c.gridwidth = GridBagConstraints.RELATIVE; //next-to-last in row
		button = new Button("Button6");
		gridbag.setConstraints(button, c);
		dialog.add(button);

		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		button = new Button("Button7");
		gridbag.setConstraints(button, c);
		dialog.add(button);

		// First column in the last row
		c.gridwidth = 1;	   	   //reset to the default
		c.gridheight = 2;
		c.weighty = 1.0;
		button = new Button("Button8");
		gridbag.setConstraints(button, c);
		dialog.add(button);

		// Second column in the last row that has two buttons
		c.weighty = 0.0;		   //reset to the default
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		c.gridheight = 1;		   //reset to the default
		button = new Button("Button9");
		gridbag.setConstraints(button, c);
		dialog.add(button);
		button = new Button("Button10");
		gridbag.setConstraints(button, c);
		dialog.add(button);
		
		dialog.setSize(300, 200);
		dialog.addWindowListener(this);
		dialog.setVisible(true);
	}

	void commandCardLayout()
	{
		// Dialog Grid
		Dialog dialog = new Dialog(m_frame, "Card Layout Dialog");
		CardLayout layout = new CardLayout(10, 10);
		dialog.setLayout(layout);
		dialog.add("1", new Label("1"));
		dialog.add("2", new Button("2"));
		dialog.add("3", new Label("3"));
		dialog.add("4", new Button("4"));
		dialog.add("5", new Label("5"));
		dialog.add("6", new Button("6"));
		dialog.setSize(300, 400);
		dialog.addWindowListener(this);
		dialog.setVisible(true);
	}

	void commandWindow()
	{
		// Window
		Window window = new Window(m_frame);
		window.setBounds(0, 0, 200, 300);
		TextArea textArea = new TextArea();
		Button button = new Button("Close");
		button.addActionListener(this);
		button.setActionCommand("closeW");
		window.add("North", button);
		window.add("Center", textArea);
		window.addWindowListener(this);
		window.show();
		MenuBar menu = createMenuBar();
		m_frame.setMenuBar(menu);
	}

	// WindowAdapter
	public void windowClosing(WindowEvent event)
	{
		Window window = event.getWindow();
		if (window == m_frame)
			System.exit(0);
		else
			window.dispose();
	}

	// ActionListener
	public void actionPerformed(ActionEvent event)
	{
		String command = event.getActionCommand();
		showString("Command : " + command);
		// File
		if (command.equals("open"))
		{
			FileDialog loadDialog = new FileDialog(m_frame, "File Open Dialog", FileDialog.LOAD);
			loadDialog.setVisible(true);
		}
		else if (command.equals("save"))
		{
			FileDialog saveDialog = new FileDialog(m_frame, "File Save Dialog", FileDialog.SAVE);
			saveDialog.setVisible(true);
		}
		// Window
		else if (command.equals("gridL"))
		{
			commandGridLayout();
		}
		else if (command.equals("gridB"))
		{
			commandGridBagLayout();
		}
		else if (command.equals("cardL"))
		{
			commandCardLayout();
		}
		// Dialogs
		else if (command.equals("myDialog"))
		{
			MyDialog dlg = new MyDialog();
			dlg.doBrowse(m_frame);
			showString(dlg.m_text);
		}
		else if (command.equals("signalsScale"))
		{
			PlotCtrlDlg dlg = new PlotCtrlDlg();
			dlg.doBrowse(m_frame);
		}
		else if (command.equals("displaySettings"))
		{
			DisplaySettingsDlg dlg = new DisplaySettingsDlg();
			dlg.doModal(m_frame);
		}
		else if (command.equals("editSourceFile"))
		{
			EditSourceFileDlg dlg = new EditSourceFileDlg();
			dlg.doModal(m_frame);
		}
		// Menus
		else if (command.equals("shellMenu"))
		{
			Frame frame = new Frame("shellMenu Frame");
			MenuBar menuBar = ShellMenu.init();
			frame.setMenuBar(menuBar);
			frame.setBounds(250, 250, 400, 100);
			frame.addWindowListener(this);
			frame.show();
		}
		// File
		else if (command.equals("closeW"))
		{
			Object object = event.getSource();
			if (object instanceof Button)
			{
				Container parent = ((Button)object).getParent();
				if (parent instanceof Window)
				{
					((Window)parent).dispose();
				}
			}
		}
		else if (command.equals("window"))
		{
			commandWindow();
		}
		else if (command.equals("exit"))
		{
			System.exit(0);
		}
	}
}
